/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_ATMega32.h                       *
*             Atmel AVR ATMega32 Specific Code             *
*                    LARGE MEMORY MODEL                    *
***********************************************************/
#ifndef _OS_ATMega32_
 #define _OS_ATMega32_

 #ifdef __IMAGECRAFT__
  #include <iom32v.h>                          /* from ICC */
  #include <macros.h>
 #elif __IAR_SYSTEMS_ICC__
  __root void OSTickISR2(void);
  __root void OSIntExit(void);
  #include <iom32.h>                           /* from IAR */
  #include <ina90.h>
 #elif __GNUC__                             /* from WinAVR */
  #include <avr/io.h>
 #elif __STDC__
  #include <avr.h>                      /* from CrossWorks */
  #include <ina90.h>
 #endif

/*
************************************************************
*           SPECIAL FOR RETURN- AND PARAM- STACK
************************************************************
*/
 #define   OS_HW_STACKSIZE     64   /* call-deep 32        */
/*
************************************************************
*               MACROS (compiler spezific)
************************************************************
*/
 #define  OS_CRITICAL_SR         0                          // do not save the CPU-register during INT-disable

 #ifdef __IMAGECRAFT__
  #define  OS_ENTER_CRITICAL()  CLI()
  #define  OS_EXIT_CRITICAL()   SEI()
 #elif __IAR_SYSTEMS_ICC__
  #define  OS_ENTER_CRITICAL()  _CLI()
  #define  OS_EXIT_CRITICAL()   _SEI()
 #elif __GNUC__
  #define  OS_ENTER_CRITICAL()  __asm__ __volatile__ ("cli" ::)
  #define  OS_EXIT_CRITICAL()   __asm__ __volatile__ ("sei" ::)
 #elif __STDC__
  #define  OS_ENTER_CRITICAL()  _CLI()
  #define  OS_EXIT_CRITICAL()   _SEI()
 #endif
 #define   OS_TASK_SW()         OSCtxSw()
/*
************************************************************
*                    ENDIAN CONVENTION
************************************************************
*/
 #define  LITTLE_ENDIAN   3412
 #define  BIG_ENDIAN      1234

 #define  BYTE_ORDER  BIG_ENDIAN      /* endianess of CPU */

//------------ for little-endian protocols -----------------
 #if BYTE_ORDER == LITTLE_ENDIAN                           // for example: USB
  #define L_htonl(n)       (n)
  #define L_htons(n)       (n)
 #else
  #define L_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define L_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  L_ntohl          L_htonl
 #define  L_ntohs          L_htons
//-------------- for big-endian protocols ------------------
 #if BYTE_ORDER == BIG_ENDIAN                              // for example: 802.3 , TCP/IP
  #define B_htonl(n)       (n)
  #define B_htons(n)       (n)
 #else
  #define B_htonl(n)       (((n & 0xff000000UL) >> 24) | ((n & 0x00ff0000UL) >> 8) | ((n & 0x0000ff00UL) << 8) | ((n & 0x000000ffUL) << 24))
  #define B_htons(n)       (((n & 0xff00U) >> 8) | ((n & 0x00ffU) << 8))
 #endif
 #define  B_ntohl          B_htonl
 #define  B_ntohs          B_htons
/*
************************************************************
*                 POINTER AND REENTRANCE
************************************************************
*/
 #define  OS_EXT  extern     /* external declaration      */
 #define  OS_FAR             /* far pointer               */
 #define  OS_HUGE            /* far pointer over segments */
 #define  OS_CONST   const   /* constant data             */
 #define  OS_ROMDATA         /* area locations in ROM     */
 #define  KC51_REENTRANT                    /* not needed */
/*
************************************************************
*                      DATA TYPES
************************************************************
*/
 typedef unsigned char  BOOLEAN;
 typedef unsigned char  U08;
 typedef signed   char  S08;
 typedef unsigned int   U16;
 typedef signed   int   S16;
 typedef unsigned long  U32;
 typedef signed   long  S32;
 typedef float          FP;
 typedef U08            OS_STK_TYPE;

/*
************************************************************
*                       CONSTANTS
************************************************************
*/
 #ifndef FALSE
  #define FALSE  0
 #endif
 #ifndef TRUE
  #define TRUE   1
 #endif
 #ifndef NULL
  #define NULL   ((void *)0)
 #endif

/*
************************************************************
*                  COMPILING DEFINITIONS
************************************************************
*/
#ifndef OS_PACKED
 #define OS_PACKED
#endif
#ifndef OS_PACKED_ATTR
 #define OS_PACKED_ATTR
#endif
#ifndef OS_ALIGN_ATTR
 #define OS_ALIGN_ATTR
#endif

/************************** END ***************************/
#endif
