/***********************************************************
*                       pC/OS 1.68a                        *
*                      a FREE - RTOS                       *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      AVR32_INTs.c                        *
*           Atmel AVR32 AT32UC3A1512 Specific Code         *
*                                                          *
***********************************************************/
#include <avr32/io.h>
#ifndef __GNUC__
 #include <intrinsics.h>
#endif
#include "AVR32_INTs.h"

#if __GNUC__
 #define clz(u)              __builtin_clz(u)
#elif __ICCAVR32__
 #define clz(u)              __count_leading_zeros(u)
#endif

extern const unsigned int ipr_val[4];                      // max 4 interrupt-levels
static INT_handler INT_table[AVR32_INT_MAX_GRP][AVR32_INT_MAX_REQ];
                                                           // the handler table

INT_handler _get_interrupt_handler(unsigned int lev)
{                                                          // called from "execption.s82"
    unsigned int grp;
    unsigned int req;
    
    grp = AVR32_INTC.icr[AVR32_INTC_INT3 - lev];           // ICR0..3 are backward located in memory
    req = AVR32_INTC.irr[grp];
    return((req)?INT_table[grp][32-clz(req)-1]:0);
}

__interrupt static void INT_unhandled_handler(void)
{                                                          // default-handler
    while (1);
}

void INT_Init(void)
{                                                          // init the grp-req table and assign all to level-0
    unsigned int grp, req;

    for (grp = 0; grp < AVR32_INT_MAX_GRP; grp++) {
        for (req = 0; req < AVR32_INT_MAX_REQ; req++)
            INT_table[grp][req] = &INT_unhandled_handler;
        AVR32_INTC.ipr[grp] = ipr_val[AVR32_INTC_INT0];
    }
}

unsigned char INT_RegHandler(INT_handler handler, unsigned int int_nr, unsigned int lev)
{                                                          // write handler into table and update level
    if(((int_nr/32) >= AVR32_INT_MAX_GRP) || ((int_nr%32) >= AVR32_INT_MAX_REQ) || (lev > AVR32_INTC_INT3)) return(1);
    INT_table[int_nr/32][int_nr%32] = handler;
    AVR32_INTC.ipr[int_nr/32] = ipr_val[lev];
    return(0);
}
