/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                       OS_186_c.c                         *
*               80186/80188 Specific Code                  *
*                  SMALL MEMORY MODEL                      *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
    U16  *stk;
    U08   err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk              = (U16 *)pstk;                       /* 80186/80188 Small Model              */
        *--stk           = (U16)FP_OFF(data);
        *--stk           = (U16)FP_SEG(task);
        *--stk           = (U16)FP_OFF(task);
        *--stk           = (U16)0x0200;                       /* PSW = Int. En.                       */
        *--stk           = (U16)FP_SEG(task);
        *--stk           = (U16)FP_OFF(task);
        *--stk           = (U16)0x0000;                       /* AX = 0                               */
        *--stk           = (U16)0x0000;                       /* CX = 0                               */
        *--stk           = (U16)0x0000;                       /* DX = 0                               */
        *--stk           = (U16)0x0000;                       /* BX = 0                               */
        *--stk           = (U16)0x0000;                       /* SP = 0                               */
        *--stk           = (U16)0x0000;                       /* BP = 0                               */
        *--stk           = (U16)0x0000;                       /* SI = 0                               */
        *--stk           = (U16)0x0000;                       /* DI = 0                               */
        *--stk           = (U16)0x0000;                       /* ES = 0                               */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}
