/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                    OS_ARC_EM_a.s                         *
*                 ARC EM Specific Code                     *
*                      using MIDE                          *
***********************************************************/

// "OS_STK_CHECK"_EN must be add as preprocessor-defines
// for assembler if enabled in OS_cfg.h

// "OWN_IRQ_STACK" must be add as preprocessor-defines
// for assembler if this feature should be used

#if OS_STK_CHECK_EN && __Xstack_check
    .set AUX_USTACK_TOP  , 0x260
    .set AUX_USTACK_BASE , 0x261
    .set AUX_KSTACK_TOP  , 0x264
    .set AUX_KSTACK_BASE , 0x265
#endif

    .set AUX_IRQ_HINT    , 0x201
    .set OS_SW_IRQ       , 18      // a valid but not assigned interrupt nr.

    .extern  OSTCBCur
    .extern  OSTCBHighRdy
    .extern  OSIntNesting
    .extern  OSTickISR

    .extern _estack
    .extern _STACKSIZE

/***********************************************************
*  this port requires setting .LP, .U, .L, .B and .NR in   *
*              AUX register "AUX_IRQ_CTRL" !               *
***********************************************************/
    .macro store_extensions
        // add here the PUSH of additional registers of user-extensions like DSP, FPU, ...
    .endm

    .macro restore_extensions
        // add here the POP of additional registers of user-extensions like DSP, FPU, ...
    .endm

    .macro store_stack_lim                    // use stack-check scenario 3
        lr      r9, [AUX_KSTACK_TOP]
        st.a    r9, [sp, -4]
        lr      r9, [AUX_KSTACK_BASE]
        st.a    r9, [sp, -4]
    .endm

    .macro restore_stack_lim                  // use stack-check scenario 1
        ld      r0, [r1]
        ld.ab   r9, [r0, 4]
        sr      r9, [AUX_KSTACK_BASE]
        ld.ab   r9, [r0, 4]
        sr      r9, [AUX_KSTACK_TOP]
        mov     sp, r0
    .endm

    .data
    .align 4
start_f:
    .word 0

#if OWN_IRQ_STACK
istack:
    .word 0
 #if OS_STK_CHECK_EN && __Xstack_check
istack_top:
    .word 0
istack_base:
    .word 0
 #endif
#endif

    .text
    .align 4

//***************************************************************
//                     START MULTITASKING
//                  void OSStartHighRdy(void)
//***************************************************************
	.global OSStartHighRdy
OSStartHighRdy:
        mov     r0, 0
        st      r0, [start_f]
#if OWN_IRQ_STACK
        mov     r0, sp
        st      r0, [istack]
 #if OS_STK_CHECK_EN && __Xstack_check
        mov     r0, _estack
        st      r0, [istack_base]
        sub     r0, r0, _STACKSIZE
        st      r0, [istack_top]
 #endif
#endif
        // NO other interrupt may be enabled here !

        seti    12
        mov     r0, OS_SW_IRQ
        sr      r0, [AUX_IRQ_HINT]
        j       [blink]

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From task level)
//                      void OSCtxSw(void)
//***************************************************************
	.global OSCtxSw
OSCtxSw:
        push    r0
        seti    12
        mov     r0, OS_SW_IRQ
        sr      r0, [AUX_IRQ_HINT]
        pop     r0
        j       [blink]

//***************************************************************
//           PERFORM A CONTEXT SWITCH (From an ISR)
//                    void OSIntCtxSw(void)
//***************************************************************
	.global OSIntCtxSw
OSIntCtxSw:
        mov     r1, OSTCBHighRdy
        ld      r1, [r1]
        mov     r0, OSTCBCur
        st      r1, [r0]
#if OS_STK_CHECK_EN && __Xstack_check
        restore_stack_lim
#else
        ld      sp, [r1]
#endif
        restore_extensions
        rtie

//***************************************************************
//                    HANDLE CONTEXT-SWITCH
//                   void OSPendSV_ISR(void)
//***************************************************************
    .global OSPendSV_ISR
OSPendSV_ISR:
        mov     r0, 0
        sr      r0, [AUX_IRQ_HINT]

        ld      r0, [start_f]
        cmp     r0, 0
        jle     OSPendSV_nosave

        store_extensions
#if OS_STK_CHECK_EN && __Xstack_check
        store_stack_lim
#endif
        mov     r0, OSTCBCur
        ld      r1, [r0]
        st      sp, [r1]
        mov     r2, OSTCBHighRdy
        ld      r3, [r2]
        st      r3, [r0]
        b       OSPendSV_save

OSPendSV_nosave:
        mov     r0, 1
        st      r0, [start_f]
OSPendSV_save:
        mov     r0, OSTCBHighRdy
        ld      r1, [r0]
#if OS_STK_CHECK_EN && __Xstack_check
        restore_stack_lim
#else
        ld      sp, [r1]
#endif
        restore_extensions
        rtie

//***************************************************************
//                       HANDLE TICK ISR
//                     void OSTick_ISR(void)
//***************************************************************
    .global OSTick_ISR
OSTick_ISR:
        store_extensions
        mov     r0, OSIntNesting
        ldb     r1, [r0]
        cmp     r1, 0
        jlne    OSTickLISR1

#if OS_STK_CHECK_EN && __Xstack_check
        store_stack_lim
#endif
        mov     r2, OSTCBCur
        ld      r3, [r2]
        st      sp, [r3]
#if OWN_IRQ_STACK
        ld      sp, [istack]
 #if OS_STK_CHECK_EN && __Xstack_check
        ld      r2, [istack_base]
        sr      r2, [AUX_KSTACK_BASE]
        ld      r2, [istack_top]
        sr      r2, [AUX_KSTACK_TOP]
 #endif
#endif
OSTickLISR1:
        adc     r1, r1, 1
        stb     r1, [r0]

        bl      OSTickISR                 // call IRQ handler

        bl      OSIntExit
        mov     r0, OSIntNesting
        ldb     r1, [r0]
        cmp     r1, 0
        jlne    OSTickLISR2

        mov     r0, OSTCBCur
        ld      r1, [r0]
#if OS_STK_CHECK_EN && __Xstack_check
        restore_stack_lim
#else
        ld      sp, [r1]
#endif
OSTickLISR2:
        restore_extensions
        rtie

//***************************************************************
//                    SPECIAL IRQ-ISR (ALL)
//                     void OSirqISR(void)
//***************************************************************
//                       not needed here
//***************************************************************


//***************************************************************
//                    SPECIAL INT-SAVE CALLS
//                  OS_STK_TYPE OSEnterCrit(void)
//                  void OSExitCrit(OS_STK_TYPE)
//***************************************************************
//                       not needed here
//***************************************************************

//***************************************************************


