;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                    OS_AT91SAM7xxx_a.s                        *
;*                  AT91-ARM Specific Code                      *
;*                using Keil (Keil-compiler)                    *
;****************************************************************
            THUMB_MODE EQU 1

            EXTERN  DATA   (OSTCBCur)
            EXTERN  DATA   (OSTCBHighRdy)
IF (THUMB_MODE != 0)
            EXTERN  CODE16 (OSIntExit?T)
ELSE
            EXTERN  CODE32 (OSIntExit?A)
ENDIF
            EXTERN  DATA   (OSIntNesting)

NO_INT      EQU     0xC0                    ; mask to disable interrupts
FIQ32_MODE  EQU     0x11
IRQ32_MODE  EQU     0x12
SVC32_MODE  EQU     0x13

AIC_IVR     EQU     0xFFFFF100              ; IRQ vector register
AIC_FVR     EQU     0xFFFFF104              ; FIQ vector register
AIC_EOICR   EQU     0xFFFFF130              ; EOI command register

        AREA PCOS, CODE, READONLY, ALIGN=4
;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
        PUBLIC  OSStartHighRdy?T , OSStartHighRdy?A    
OSStartHighRdy?T   PROC CODE16              ; thumb interworking entry
        BX      PC                          ; branch to 'OSStartHighRdy?A'
        NOP
        ENDP

OSStartHighRdy?A   PROC CODE32              ; arm entry !
        MSR     CPSR_cxsf, #0xD3            ; switch to SVC mode with IRQ and FIQ disabled
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore stack-ptr
        LDR     R4, [SP], #4                ; pop task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop task context
        ENDP

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
        PUBLIC  OSCtxSw?T , OSCtxSw?A
OSCtxSw?T          PROC CODE16              ; thumb interworking entry
        BX      PC                          ; branch to 'OSCtxSw?A'
        NOP
        ENDP

OSCtxSw?A          PROC CODE32              ; arm entry !
        STMFD   SP!, {LR}                   ; push return address
        STMFD   SP!, {LR}
        STMFD   SP!, {R0-R12}               ; push registers
        MRS     R4, CPSR                    ; push current CPSR
        TST     LR, #1                      ; see if called from Thumb mode
        ORRNE   R4, R4, #0x20               ; if yes, Set the T-bit
        STMFD   SP!, {R4}
        LDR     R4, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore new stack-ptr
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        ENDP

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
        PUBLIC  OSIntCtxSw?T , OSIntCtxSw?A
OSIntCtxSw?T       PROC CODE16              ; thumb interworking entry
        BX      PC                          ; branch to 'OSIntCtxSw?A'
        NOP
        ENDP
		                                    ; no TCB-save since the INTs can cascaded
OSIntCtxSw?A       PROC CODE32              ; (only saved on first INT by OSirqISR or OSfiqISR)
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore new stack-ptr
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        ENDP

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
;   ==> this is not asm-relevant for this processor
;       the C-TimerISR is to hang into the table of
;       the following IRQ-isr
;****************************************************************


;****************************************************************
;                     SPECIAL IRQ-ISR (ALL)
;                      void OSirqISR(void)
;****************************************************************
        PUBLIC  OSirqISR?A
OSirqISR?A		   PROC CODE32
        STMFD   SP!, {R1-R3}                ; push working registers onto IRQ stack
        MOV     R1, SP                      ; save IRQ stack pointer
        ADD     SP, SP,#12                  ; adjust IRQ stack pointer
        SUB     R2, LR,#4                   ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from IRQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1 ...
        BNE     OSirqISR1
        LDR     R4, =OSTCBCur               ; ... Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSirqISR1:
        MSR     CPSR_c, #(NO_INT | IRQ32_MODE) ; change to IRQ mode (to use the IRQ stack to handle interrupt)

        LDR     R1, =AIC_IVR                ; branch to the routine pointed by the AIC_IVR
        LDR     R0, [R1]
        STR     R1, [R1]
        MOV     LR, PC
        BX      R0

        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        LDR     R0, =AIC_EOICR              ; mark the End-Of-Interrupt on the AIC
        STR     R0, [R0]
IF (THUMB_MODE != 0)
        LDR     R0, =OSIntExit?T            ; thumb-mode, call OSIntExit() -> perform a context-switch if nesesary
ELSE
        LDR     R0, =OSIntExit?A            ; arm-mode, call OSIntExit() -> perform a context-switch if nesesary
ENDIF
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        ENDP

;****************************************************************
;                     SPECIAL FIQ-ISR (ALL)
;                      void OSfiqISR(void)
;****************************************************************
        PUBLIC  OSfiqISR?A
OSfiqISR?A		   PROC CODE32
        STMFD   SP!, {R1-R3}                ; push working registers onto FIQ stack
        MOV     R1, SP                      ; save FIQ stack pointer
        ADD     SP, SP,#12                  ; adjust FIQ stack pointer
        SUB     R2, LR,#4                   ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from FIQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1
        BNE     OSfiqISR1
        LDR     R4, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSfiqISR1:
        MSR     CPSR_c, #(NO_INT | FIQ32_MODE) ; change to FIQ mode (to use the IRQ stack to handle interrupt)

        LDR     R1, =AIC_FVR                ; branch to the routine pointed by the AIC_FVR
        LDR     R0, [R1]
        STR     R1, [R1]
        MOV     LR, PC
        BX      R0

        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode

; end-of-interrupt to add !

IF (THUMB_MODE != 0)
        LDR     R0, =OSIntExit?T            ; thumb-mode, call OSIntExit() -> perform a context-switch if nesesary
ELSE
        LDR     R0, =OSIntExit?A            ; arm-mode, call OSIntExit() -> perform a context-switch if nesesary
ENDIF
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        ENDP


        END


