;****************************************************************
;*                       pC/OS 1.68a                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                     OS_ATMega128_a.s                         *
;*                     AVR Specific Code                        *
;*                        using ICC                             *
;****************************************************************
SREG    = 0x3F
SPH     = 0x3E
SPL     = 0x3D
RAMPZ   = 0x3B
TCNT0   = 0x52

;****************************************************************
; The following macro is the assembly code to save the current 
; task context
        .macro  save_context
        ST      -Y,R0
        ST      -Y,R1
        ST      -Y,R2
        ST      -Y,R3
        ST      -Y,R4
        ST      -Y,R5
        ST      -Y,R6
        ST      -Y,R7
        ST      -Y,R8
        ST      -Y,R9
        ST      -Y,R10
        ST      -Y,R11
        ST      -Y,R12
        ST      -Y,R13
        ST      -Y,R14
        ST      -Y,R15
        ST      -Y,R16
        ST      -Y,R17
        ST      -Y,R18
        ST      -Y,R19
        ST      -Y,R20
        ST      -Y,R21
        ST      -Y,R22
        ST      -Y,R23
        ST      -Y,R24
        ST      -Y,R25
        ST      -Y,R26
        ST      -Y,R27
        ST      -Y,R30
        ST      -Y,R31
        IN      R16,RAMPZ
        ST      -Y,R16
        IN      R16,SREG
        ST      -Y,R16
        IN      R16,SPH
        ST      -Y,R16
        IN      R16,SPL
        ST      -Y,R16
        .endmacro

;****************************************************************
; The following macro is the assembly code to restore the current 
; task context
        .macro  restore_context
        LD      R16,Y+
        OUT     SPL,R16
        LD      R16,Y+
        OUT     SPH,R16
        LD      R16,Y+
        OUT     SREG,R16
        LD      R16,Y+
        OUT     RAMPZ,R16
        LD      R31,Y+
        LD      R30,Y+
        LD      R27,Y+
        LD      R26,Y+
        LD      R25,Y+
        LD      R24,Y+
        LD      R23,Y+
        LD      R22,Y+
        LD      R21,Y+
        LD      R20,Y+
        LD      R19,Y+
        LD      R18,Y+
        LD      R17,Y+
        LD      R16,Y+
        LD      R15,Y+
        LD      R14,Y+
        LD      R13,Y+
        LD      R12,Y+
        LD      R11,Y+
        LD      R10,Y+
        LD      R9,Y+
        LD      R8,Y+
        LD      R7,Y+
        LD      R6,Y+
        LD      R5,Y+
        LD      R4,Y+
        LD      R3,Y+
        LD      R2,Y+
        LD      R1,Y+
        LD      R0,Y+
        .endmacro

		.area   text(rel)
;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
_OSStartHighRdy::
        LDS     R30,_OSTCBHighRdy           ; Let Z point to TCB of highest priority task
        LDS     R31,_OSTCBHighRdy+1         ; ready to run
        STS     _OSTCBCur,R30               ; save as current
        STS     _OSTCBCur+1,R31             ;
        LD      R28,Z+                      ; Load Y (R29:R28) pointer
        LD      R29,Z+                      ;
        restore_context                     ; restore stack-ptr, status, registers
        RET                                 ; Start task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
_OSCtxSw::
        save_context                        ; save registers, status, stack-ptr
        LDS     R30,_OSTCBCur               ; Save stack ptr in TCB
        LDS     R31,_OSTCBCur+1             ;
        ST      Z+,R28                      ; Save Y (R29:R28) pointer
        ST      Z+,R29                      ;
        LDS     R30,_OSTCBHighRdy           ; Let Z point to TCB of highest priority task
        LDS     R31,_OSTCBHighRdy+1         ; ready to run
        STS     _OSTCBCur,R30               ; OSTCBCur = OSTCBHighRdy
        STS     _OSTCBCur+1,R31             ;
        LD      R28,Z+                      ; Restore Y pointer
        LD      R29,Z+                      ;
        restore_context                     ; restore stack-ptr, status, registers
        RET                                 ; Return to new task

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
_OSIntCtxSw::
        LDS     R30,_OSTCBHighRdy           ; Z = OSTCBHighRdy->OSTCBStkPtr
        LDS     R31,_OSTCBHighRdy+1         ;
        STS     _OSTCBCur,R30               ; OSTCBCur = OSTCBHighRdy
        STS     _OSTCBCur+1,R31             ;
        LD      R28,Z+                      ; Restore Y pointer
        LD      R29,Z+                      ;
        restore_context                     ; restore stack-ptr, status, registers
        RETI                                ; Return to new task

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
_OSTickISR::
        save_context                        ; save registers, status, stack-ptr
        LDS     R30,_OSIntNesting           ; register the ISR-level  
        INC     R30                         ; (do not call OSIntEnter ! - since the..
        STS     _OSIntNesting,R30           ;  ..INTs can be reenabled inside an ISR)
        CPI     R30,1                       ; if OSIntNesting == 1 ..
        BRNE    OSTickISR_1                 ;
        LDS     R30,_OSTCBCur               ; .. save stack ptr in TCB
        LDS     R31,_OSTCBCur+1             ;
        ST      Z+,R28                      ; Save Y (R29:R28) pointer
        ST      Z+,R29                      ;
OSTickISR_1:
        CALL    _OSTickISR2                 ; CALL OS-TIMER FUNCTION
        CALL    _OSIntExit                  ; perform a context-switch if nesesary
        restore_context                     ; restore stack-ptr, status, registers
        RETI                                ; Return to task



