//****************************************************************
//*                       pC/OS 1.70b                            *
//*                      a FREE - RTOS                           *
//*                                                              *
//*                 based on the C/OS 1.00                      *
//*                   by  Jean J. Labrosse                       *
//*                                                              *
//*    changed by Frank Goetze  -  www.embedded-os.de            *
//****************************************************************
//*                     OS_AVR32_a.asm                           *
//*                   AVR32 Specific Code                        *
//*                        using GCC                             *
//****************************************************************
        .extern OSTCBCur
        .extern OSTCBHighRdy
        .extern OSIntExit
        .extern OSIntNesting
        .extern OSTickISR_CB


        .equ AVR32_SR_OFFSET,      0    // Status  Register offset in System Register
        .equ AVR32_SR_GM_OFFSET,  16    // Status  Register, Global Interrupt Mask Offset
        .equ AVR32_SR_MB2_OFFSET, 24    // Status  Register, ModeBit-2 Offset


        .file    "OS_CPU_A"
        .section .text, "ax"
//****************************************************************
//                      START MULTITASKING
//                   void OSStartHighRdy(void)
//****************************************************************
        .global OSStartHighRdy
OSStartHighRdy:
        MOV     R12, LO(OSTCBHighRdy)       // get TCB of highest priority task
        ORH     R12, HI(OSTCBHighRdy)
        LD.W    R10, R12[0]
        MOV     R8,  LO(OSTCBCur)           // save as current
        ORH     R8,  HI(OSTCBCur)
        ST.W    R8[0], R10
        LD.W    SP, R10[0]                  // restore stack-ptr
        SSRF    AVR32_SR_MB2_OFFSET         // set status as interrupt (simulate return from interrupt)
        POPM    R0-R7                       // pop task R0-R7
        RETE                                // pop task context

//****************************************************************
//            PERFORM A CONTEXT SWITCH (From task level)
//                       void OSCtxSw(void)
//****************************************************************
        .global OSCtxSwS
OSCtxSwS:                                   // simulate interrupt stack
        PUSHM   R10-R12, LR                 // save task R10-R12, LR
        LD.D    R10, SP[4*4]                // copy task SR and PC from bottom of stack (from call) ...
        PUSHM   R10-R11                     // ... to actual stack position
        ST.D    SP[6*4], R8                 // push task R8 and R9 onto bottom of stack
        PUSHM   R0-R7                       // push task R0-R7
        MOV     R8, LO(OSTCBCur)            // save stack ptr in TCB
        ORH     R8, HI(OSTCBCur)
        LD.W    R9, R8[0]
        ST.W    R9[0], SP
        MOV     R12, LO(OSTCBHighRdy)       // get TCB of highest priority task
        ORH     R12, HI(OSTCBHighRdy)
        LD.W    R10, R12[0]
        MOV     R8, LO(OSTCBCur)            // save as current
        ORH     R8, HI(OSTCBCur)
        ST.W    R8[0], R10
        LD.W    SP, R10[0]                  // restore new stack-ptr
        SSRF    AVR32_SR_MB2_OFFSET         // set status as interrupt (simulate return from interrupt)
        POPM    R0-R7                       // pop new task R0-R7
        RETE                                // pop new task context

//****************************************************************
//            PERFORM A CONTEXT SWITCH (From an ISR)
//                     void OSIntCtxSw(void)
//****************************************************************
        .global OSIntCtxSw
OSIntCtxSw: 
        MOV     R12, LO(OSTCBHighRdy)       // get TCB of highest priority task
        ORH     R12, HI(OSTCBHighRdy)
        LD.W    R10, R12[0]
        MOV     R8,  LO(OSTCBCur)           // save as current
        ORH     R8,  HI(OSTCBCur)
        ST.W    R8[0], R10
        LD.W    SP, R10[0]                  // restore new stack-ptr
        POPM    R0-R7                       // pop new task R0-R7
        RETE                                // pop new task context

//****************************************************************
//                     HANDLE TICK ISR (ENTRY)
//                      void OSTickISR(void)
//****************************************************************
        .global OSTickISR
OSTickISR:
        SSRF    AVR32_SR_GM_OFFSET          // disable interrupts
        PUSHM   R0-R7                       // push task R0-R7
        MOV     R11, LO(OSIntNesting)       // OSIntNesting++
        ORH     R11, HI(OSIntNesting)
        LD.UB   R10, R11[0]
        SUB     R10, -1
        ST.B    R11[0], R10
        CP.W    R10, 1                      // if OSIntNesting == 1 ...
        BRNE    OSTickISR_1
        MOV     R11, LO(OSTCBCur)           // ... save stack ptr in TCB
        ORH     R11, HI(OSTCBCur)
        LD.W    R10, R11[0]
        ST.W    R10[0], SP
OSTickISR_1:
        CSRF    AVR32_SR_GM_OFFSET          // enable interrupts
        RCALL   OSTickISR_CB                // call user ISR function (neesting possible)
        SSRF    AVR32_SR_GM_OFFSET          // disable interrupts
        RCALL   OSIntExit                   // call OSIntExit() -> perform a context-switch if nesesary
        POPM    R0-R7                       // pop task R0-R7
        RETE                                // pop task context

//****************************************************************
//                     SPECIAL INT-SAVE CALLS
//                   OS_STK_TYPE OSEnterCrit(void)
//                   void OSExitCrit(OS_STK_TYPE)
//****************************************************************
        .global OSEnterCrit
OSEnterCrit:
        CSRFCZ  AVR32_SR_GM_OFFSET          // get GM bit from SR
        SRCS    R12                         // if set, set R12 as return
        SSRF    AVR32_SR_GM_OFFSET          // disable interrupts
        NOP
        NOP
        MOV     PC, LR                      // return

        .global OSExitCrit
OSExitCrit:
        PUSHM   R11                         // save R11 into stack
        MFSR    R11, AVR32_SR_OFFSET        // get current SR
        LSR     R12, 1                      // copy given interrupt status to Carry
        BST     R11, AVR32_SR_GM_OFFSET     // overwrite GM bit based on Carry
        MTSR    AVR32_SR_OFFSET, R11        // restore SR with GM bit of given status
        POPM    R11                         // restore R11 from stack
        MOV     PC, LR                      // return


//****************************************************************



