/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     trap_FE310.c                         *
*               FE310 RISC-V Specific Code                 *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

typedef void (*vectors_t)(void);
vectors_t vectors[52] = {
        NULL,       // ZERO
        NULL,       //  1 - watchdog
        NULL,       //  2 - RTC
        NULL,       //  3 - UART0
        NULL,       //  4 - UART1
        NULL,       //  5 - QSPI0
        NULL,       //  6 - QSPI1
        NULL,       //  7 - QSPI2
        NULL,       //  8 - GPIO  0
        NULL,       //  9 - GPIO  1
        NULL,       // 10 - GPIO  2
        NULL,       // 11 - GPIO  3
        NULL,       // 12 - GPIO  4
        NULL,       // 13 - GPIO  5
        NULL,       // 14 - GPIO  6
        NULL,       // 15 - GPIO  7
        NULL,       // 16 - GPIO  8
        NULL,       // 17 - GPIO  9
        NULL,       // 18 - GPIO 10
        NULL,       // 19 - GPIO 11
        NULL,       // 20 - GPIO 12
        NULL,       // 21 - GPIO 13
        NULL,       // 22 - GPIO 14
        NULL,       // 23 - GPIO 15
        NULL,       // 24 - GPIO 16
        NULL,       // 25 - GPIO 17
        NULL,       // 26 - GPIO 18
        NULL,       // 27 - GPIO 19
        NULL,       // 28 - GPIO 20
        NULL,       // 29 - GPIO 21
        NULL,       // 30 - GPIO 22
        NULL,       // 31 - GPIO 23
        NULL,       // 32 - GPIO 24
        NULL,       // 33 - GPIO 25
        NULL,       // 34 - GPIO 26
        NULL,       // 35 - GPIO 27
        NULL,       // 36 - GPIO 28
        NULL,       // 37 - GPIO 29
        NULL,       // 38 - GPIO 30
        NULL,       // 39 - GPIO 31
        NULL,       // 40 - PWM0CMP0
        NULL,       // 41 - PWM0CMP1
        NULL,       // 42 - PWM0CMP2
        NULL,       // 43 - PWM0CMP3
        NULL,       // 44 - PWM1CMP0
        NULL,       // 45 - PWM1CMP1
        NULL,       // 46 - PWM1CMP2
        NULL,       // 47 - PWM1CMP3
        NULL,       // 48 - PWM2CMP0
        NULL,       // 49 - PWM2CMP1
        NULL,       // 50 - PWM2CMP2
        NULL        // 51 - PWM2CMP3
};

void handle_plic(void)
{
    U32 i = 0;
    unsigned long long pending;

    pending = ((unsigned long long)PLIC->pending[1] << 32) | PLIC->pending[0];
    while(i < 52) {
        if((pending & ((unsigned long long)1 << i))) {
            if(vectors[i] != NULL) vectors[i]();
            PLIC->claim = i;
        }
        i++;
    }
}

void handle_trap(U32 mcause)
{
    volatile uint32_t lpc;

    switch(mcause) {
    case mcause_exc_iam:                // instruction misaligned
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_iaf:                // instruction access fault
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_ii:                 // illegal instruction
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_break:              // breakpoint
//        while(1);
        break;
    case mcause_exc_lam:                // load address misaligned
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_laf:                // load address fault
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_sam:                // store/AMO address misaligned
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_saf:                // store/AMO access fault
    	lpc = read_csr(mepc);
        while(1);
        break;
    case mcause_exc_ecfM:               // environment call from M-mode
//        while(1);
        break;
    case mcause_int_msi:                // machine software interrupt
        while(1);                       // --> managed by OSirqISR (OS_FE310_a.S)
        break;
    case mcause_int_tsi:                // machine timer interrupt
        OSTickISR();                    // --> kernel tick handler
        break;
    case mcause_int_mei:                // machine external interrupt -> see PLIC interrupts
        handle_plic();
        break;
    default:
        break;
    }

}

/**********************************************************/
