/***************************************************************
 *                       pC/OS 1.70b                           *
 *                      a FREE - RTOS                          *
 *                                                             *
 *                 based on the C/OS 1.00                     *
 *                   by  Jean J. Labrosse                      *
 *                                                             *
 *    changed by Frank Goetze  -  www.embedded-os.de           *
 ***************************************************************
 *                     OS_MSP430_a.s43                         *
 *                TI MSP430 Specific Code                      *
 *                       using GCC                             *
 **************************************************************/
 #include <msp430x16x.h>

    .extern OSTCBHighRdy
    .extern OSTCBCur
    .extern OSIntNesting
    .extern OSTickISR
    .extern OSIntExit

    .global OSStartHighRdy
    .global OSCtxSw
    .global OSIntCtxSw
    .global OSEnterCrit
    .global OSExitCrit

    .section    .data
OSISRStkPtr:
    .word 1

    .section    .text
/***************************************************************
 *                         marcos
 **************************************************************/
    .macro PUSHALL
        push     r4
        push     r5
        push     r6
        push     r7
        push     r8
        push     r9
        push     r10
        push     r11
        push     r12
        push     r13
        push     r14
        push     r15
    .endm

    .macro POPALL
        pop      r15
        pop      r14
        pop      r13
        pop      r12
        pop      r11
        pop      r10
        pop      r9
        pop      r8
        pop      r7
        pop      r6
        pop      r5
        pop      r4
    .endm

/***************************************************************
 *                     START MULTITASKING
 *                  void OSStartHighRdy(void)
 **************************************************************/
OSStartHighRdy:
        mov.w     R1, &OSISRStkPtr      ; save interrupt stack
        mov.w     &OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w     @R13, R1              ; restore stack-ptr
        POPALL
        reti                            ; emulate return from interrupt

/***************************************************************
 *           PERFORM A CONTEXT SWITCH (From task level)
 *                      void OSCtxSw(void)
 **************************************************************/
OSCtxSw:
        push      R2                    ; emulate interrupt by also saving the SR
        PUSHALL                         ; push all registers
        mov.w     &OSTCBCur, R13        ; Save stack ptr in TCB
        mov.w     R1, 0(R13)
        mov.w     &OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w     @R13, R1              ; restore new stack-ptr
        POPALL
        reti                            ; emulate return from interrupt

/***************************************************************
 *           PERFORM A CONTEXT SWITCH (From an ISR)
 *                   void OSIntCtxSw(void)
 **************************************************************/
OSIntCtxSw:
        mov.w     &OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &OSTCBCur        ; save as current
        mov.w     @R13, R1              ; restore new stack-ptr
        POPALL
        reti                            ; return from interrup

/***************************************************************
 *                       HANDLE TICK ISR
 *                     void OSTickISR(void)
 **************************************************************/
WDT_ISR:                                ; wd timer ISR
        PUSHALL
        bic.b    #0x01, IE1             ; disable wd-timer interrupt
        cmp.b    #0, &OSIntNesting      ; if (OSIntNesting == 0)
        jne      WDT_ISR_1
        mov.w    &OSTCBCur, R13         ; save task-stack
        mov.w    R1, 0(R13)
        mov.w    &OSISRStkPtr, R1       ; load interrupt stack
WDT_ISR_1:
        inc.b    &OSIntNesting          ; increment OSIntNesting
        bis.b    #0x01, IE1             ; enable wd-timer interrupt
        EINT                            ; enable general-interrupt to allow for interrupt nesting
        call     #OSTickISR
        DINT                            ; disable general-interrupt BEFORE calling OSIntExit()
        call     #OSIntExit
        cmp.b    #0, &OSIntNesting      ; if (OSIntNesting == 0)
        jne      WDT_ISR_2
        mov.w    &OSTCBHighRdy, R13     ; restore task-stack SP
        mov.w    @R13, R1
WDT_ISR_2:
        POPALL
        reti                            ; return from interrupt

/***************************************************************
 *                    SPECIAL INT-SAVE CALLS
 *                  OS_STK_TYPE OSEnterCrit(void)
 *                  void OSExitCrit(OS_STK_TYPE)
 **************************************************************/
OSEnterCrit:
        mov.w    R2, R15
        dint
        ret

OSExitCrit:
        mov.w    R15, R2
        ret


/***************************************************************
 *               WD TIMER INTERRUPT VECTOR ENTRY
 *
 *         MSP430x11x1/MSP430F14x Interrupt vectors
 **************************************************************/
    .section    .vectors
    .org        0x14, 0xFF
    .word       WDT_ISR                 ; interrupt vector. Watchdog/Timer, Timer mode

/**************************************************************/

