/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_MSP430_c.c                        *
*                TI MSP430 Specific Code                   *
*                       using IAR                          *
***********************************************************/
#include "../../../inc/pC_OS.h"
#include "../../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U16 *stk;
    U08  err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk    = (U16 *)pstk;
        *--stk = (U16)task;
        *--stk = (U16)task;                                   /* Interrupt return pointer             */
        *--stk = (U16)0x0008;                                 /* Status register                      */
        *--stk = (U16)0x0404;
        *--stk = (U16)0x0505;
        *--stk = (U16)0x0606;
        *--stk = (U16)0x0707;
        *--stk = (U16)0x0808;
        *--stk = (U16)0x0909;
        *--stk = (U16)0x1010;
        *--stk = (U16)0x1111;
        *--stk = (U16)Tdata;                                  /* Pass 'Tdata' through register R12    */
        *--stk = (U16)0x1313;
        *--stk = (U16)0x1414;                         
        *--stk = (U16)0x1515;
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

