;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                       OS_MSP430_a.s                          *
;*                 TI MSP430 Specific Code                      *
;*                        using ICC                             *
;****************************************************************

	.area bss
_OSISRStkPtr:: 
	.word 1 
	.blkw 1 

	.area text
;****************************************************************
;                             MACROS
;****************************************************************
    .macro PUSHALL
        push     r4
        push     r5
        push     r6
        push     r7
        push     r8
        push     r9
        push     r10
        push     r11
        push     r12
        push     r13
        push     r14
        push     r15         
    .endmacro

    .macro POPALL
        pop      r15
        pop      r14
        pop      r13
        pop      r12
        pop      r11
        pop      r10
        pop      r9
        pop      r8
        pop      r7
        pop      r6
        pop      r5
        pop      r4          
    .endmacro

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
	.even
_OSStartHighRdy::
        mov.w    SP, &_OSISRStkPtr      ; save interrupt stack              
        mov.w    &_OSTCBHighRdy, R13    ; get TCB of highest priority task
        mov.w     R13, &_OSTCBCur       ; save as current
        mov.w    @R13, SP               ; restore stack-ptr 
        POPALL 
        reti                            ; emulate return from interrupt

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
	.even
_OSCtxSw::
        push      sr                    ; emulate interrupt by also saving the SR
        PUSHALL                         ; push all registers             
        mov.w     &_OSTCBCur, R13       ; Save stack ptr in TCB
        mov.w     SP, 0(R13)
        mov.w     &_OSTCBHighRdy, R13   ; get TCB of highest priority task
        mov.w     R13, &_OSTCBCur       ; save as current
        mov.w     @R13, SP              ; restore new stack-ptr
        POPALL
        reti                            ; emulate return from interrupt

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
	.even
_OSIntCtxSw::
        mov.w     &_OSTCBHighRdy, R13   ; get TCB of highest priority task
        mov.w     R13, &_OSTCBCur       ; save as current
        mov.w     @R13, SP              ; restore new stack-ptr
        POPALL
        reti                            ; return from interrup

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
	.even
_WDT_ISR::                              ; wd timer ISR
        PUSHALL             
        bic.b    #0x01, &0              ; disable wd-timer interrupt
        cmp.b    #0, &_OSIntNesting     ; if (OSIntNesting == 0)  
        jne      WDT_ISR_1
        mov.w    &_OSTCBCur, R13        ; save task-stack
        mov.w    SP, 0(R13)
        mov.w    &_OSISRStkPtr, SP      ; load interrupt stack             
WDT_ISR_1:
        inc.b    &_OSIntNesting         ; increment OSIntNesting
        bis.b    #0x01, &0              ; enable wd-timer interrupt
        EINT                            ; enable general-interrupt to allow for interrupt nesting
        call     #_OSTickISR
        DINT                            ; disable general-interrupt BEFORE calling OSIntExit()
        call     #_OSIntExit
        cmp.b    #0, &_OSIntNesting     ; if (OSIntNesting == 0) 
        jne      WDT_ISR_2
        mov.w    &_OSTCBHighRdy, R13    ; restore task-stack SP
        mov.w    @R13, SP
WDT_ISR_2:
        POPALL
        reti                            ; return from interrupt

;****************************************************************
;                     SPECIAL INT-SAVE CALLS
;                   OS_STK_TYPE OSEnterCrit(void)
;                   void OSExitCrit(OS_STK_TYPE)
;****************************************************************
	.even
_OSEnterCrit::
        mov.w    SR, R12
        dint
        ret

	.even
_OSExitCrit::
        mov.w    R12, SR
        ret


;****************************************************************
;                WD TIMER INTERRUPT VECTOR ENTRY
;                  MSP430Fxxx Interrupt vector             
;****************************************************************
	.area vector(abs)
	.org 0xFFF4                     ; Watchdog Timer
	.word _WDT_ISR                  ; interrupt vector. Watchdog/Timer, Timer mode

;****************************************************************
