/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_PhilipsXA_c.c                    *
*        Philips-XA Specific Code (FAR MEMORY MODEL)       *
*                                                          *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
    U16  *stk;
    U08   err;
    U32   temp, dtemp;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk              = (U16 *)pstk;                       /* Philips-XA Model                     */
        temp             = (U32)ttask;                        /* Convert 'task' to 32-bit value       */ 
        *--stk           = (U16)( temp & 0xFFFFL);            /* Place lower 16 bits of 'task'        */
        *--stk           = (U16)((temp >> 16) & 0x00FFL);     /* Place upper  8 bits of 'task'        */
        *--stk           = (U16)0x6666;                       /* R6   = 0x6666                        */
        *--stk           = (U16)0x5555;                       /* R5   = 0x5555                        */
        *--stk           = (U16)0x4444;                       /* R4   = 0x4444                        */
        *--stk           = (U16)0x3333;                       /* R3   = 0x3333                        */
        *--stk           = (U16)0x2222;                       /* R2   = 0x2222                        */
        dtemp            = (U32)tdata;                        /* Convert 'data' to 32-bit value       */
        *--stk           = (U16)((dtemp >> 16) & 0x00FFL);    /* R1:R0 = data (in registers R1:R0)    */
        *--stk           = (U16)(dtemp & 0xFFFFL);            /*                                      */
        *--stk           = (U16)0x0000;                       /* ES   = 0x00                          */
        *--stk           = (U16)0xFFFF;                       /* SSEL = 0xFF                          */
        *--stk           = (U16)0x0000;                       /* PSW                                  */
        *--stk           = (U16)((temp >> 16) & 0x00FFL);     /* PC(H), i.e. in reverse order.        */
        *--stk           = (U16)( temp & 0xFFFFL);            /* PC(L)                                */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}


