/***************************************************************
 *                       pC/OS 1.87b                           *
 *                      a FREE - RTOS                          *
 *                                                             *
 *                 based on the C/OS 1.00                     *
 *                   by  Jean J. Labrosse                      *
 *                                                             *
 *    changed by Frank Goetze  -  www.embedded-os.de           *
 ***************************************************************
 *                       OS_PIC32_a.s                          *
 *          PIC32MZ MIPS32 M-Class Specific Code               *
 *                       using MPLAB-X                         *
 **************************************************************/
    .global OSStartHighRdy
    .global OSCtxSw
    .global OSIntCtxSw

    .global OSEnterCrit
    .global OSExitCrit

    .extern OSTCBCur
    .extern OSTCBHighRdy
    .extern OSIntEnter
    .extern OSIntExit
    .extern OSIntNesting
    .extern OSTickISR

/* $sp = $29 */

    .text
    .set noat
    .set noreorder
/***************************************************************
 *                         marcos
 **************************************************************/
    .macro save_context
        subu  $sp, 136
        sw    $31, 132($sp)
        sw    $30, 128($sp)
        sw    $28, 124($sp)
        sw    $27, 120($sp)
        sw    $26, 116($sp)
        sw    $25, 112($sp)
        sw    $24, 108($sp)
        sw    $23, 104($sp)
        sw    $22, 100($sp)
        sw    $21, 96($sp)
        sw    $20, 92($sp)
        sw    $19, 88($sp)
        sw    $18, 84($sp)
        sw    $17, 80($sp)
        sw    $16, 76($sp)
        sw    $15, 72($sp)
        sw    $14, 68($sp)
        sw    $13, 64($sp)
        sw    $12, 60($sp)
        sw    $11, 56($sp)
        sw    $10, 52($sp)
        sw    $9, 48($sp)
        sw    $8, 44($sp)
        sw    $7, 40($sp)
        sw    $6, 36($sp)
        sw    $5, 32($sp)
        sw    $4, 28($sp)
        sw    $3, 24($sp)
        sw    $2, 20($sp)
        sw    $1, 16($sp)
        mflo  $8
        sw    $8, 8($sp)
        mfhi  $8
        sw    $8, 12($sp)
        mfc0  $8, $14, 0
        sw    $8, 4($sp)
        mfc0  $8, $12, 0
        sw    $8, 0($sp)
    .endm

    .macro restore_context
        lw    $8, 0($sp)
        mtc0  $8, $12, 0
        lw    $8, 4($sp)
        mtc0  $8, $14, 0
        lw    $8, 12($sp)
        mthi  $8
        lw    $8, 8($sp)
        mtlo  $8
        lw    $1, 16($sp)
        lw    $2, 20($sp)
        lw    $3, 24($sp)
        lw    $4, 28($sp)
        lw    $5, 32($sp)
        lw    $6, 36($sp)
        lw    $7, 40($sp)
        lw    $8, 44($sp)
        lw    $9, 48($sp)
        lw    $10, 52($sp)
        lw    $11, 56($sp)
        lw    $12, 60($sp)
        lw    $13, 64($sp)
        lw    $14, 68($sp)
        lw    $15, 72($sp)
        lw    $16, 76($sp)
        lw    $17, 80($sp)
        lw    $18, 84($sp)
        lw    $19, 88($sp)
        lw    $20, 92($sp)
        lw    $21, 96($sp)
        lw    $22, 100($sp)
        lw    $23, 104($sp)
        lw    $24, 108($sp)
        lw    $25, 112($sp)
        lw    $26, 116($sp)
        lw    $27, 120($sp)
        lw    $28, 124($sp)
        lw    $30, 128($sp)
        lw    $31, 132($sp)
        addu  $sp, 136
    .endm

/***************************************************************
 *                     START MULTITASKING
 *                  void OSStartHighRdy(void)
 **************************************************************/
    .ent OSStartHighRdy
OSStartHighRdy:
        la    $8, OSTCBHighRdy              /* get TCB of highest priority task */
        lw    $8, 0($8)
        lw    $sp, 0($8)                    /* restore stack-ptr */
        restore_context
        jr    $31
        nop
    .end OSStartHighRdy

/***************************************************************
 *         INITIATE A CONTEXT SWITCH (From task level)
 *                    void OSCtxSw(void)
 **************************************************************/
    .ent OSCtxSw
OSCtxSw:
        lui   $t0, %hi(IFS0)
        lw    $t1, %lo(IFS0)($t0)
        ori   $t1, $t1, 2                   /* set cs0 pending bit */
        sw    $t1, %lo(IFS0)($t0)
        ssnop
        jr    $ra
        nop
    .end OSCtxSw

/***************************************************************
 *            PERFORM A CONTEXT SWITCH (From an ISR)
 *                   void OSIntCtxSw(void)
 **************************************************************/
    .ent OSIntCtxSw
OSIntCtxSw:
        la    $8, OSTCBHighRdy              /* get TCB of highest priority task */
        lw    $9, 0($8)
        la    $10, OSTCBCur                 /* save as current */
        sw    $9, 0($10)
        lw    $sp, 0($9)                    /* restore new stack-ptr */
        restore_context
        eret
        nop
    .end OSIntCtxSw

/***************************************************************
 *                   HANDLE CONTEXT-SWITCH
 *           PERFORM A CONTEXT SWITCH (From task level)
 *                        IRQ-HANDLER
 **************************************************************/
    .ent OSCtxSw_ISR
OSCtxSw_ISR:
        save_context
        la    $8, OSTCBCur
        lw    $9, 0($8)
        sw    $sp, 0($9)
	
        mfc0  $8, $12, 0
        mfc0  $9, $13, 0
        andi  $9, $9, 0xFC00
        ins   $8, $0, 10, 6
        or    $8, $8, $9
        mtc0  $8, $12, 0
	
	lui   $8, %hi(IFS0CLR)              /* clear cs0 pending bit */
	ori   $9, $zero, 2
	sw    $9, %lo(IFS0CLR)($8)
	
	la    $8, OSTCBHighRdy              /* get TCB of highest priority task */
        lw    $9, 0($8)
        la    $10, OSTCBCur                 /* save as current */
        sw    $9, 0($10)
        lw    $sp, 0($9)                    /* restore new stack-ptr */
        restore_context
        eret
        nop
    .end OSCtxSw_ISR

/***************************************************************
 *                      HANDLE CORE-TIMER
 *                 PERFORM CORE-TIMER HANDLING
 *                        IRQ-HANDLER
 **************************************************************/
    .ent OSTick_ISR
OSTick_ISR:
        save_context
        la    $8, OSIntNesting              /* if OSIntNesting == 0 */
        lbu   $9, 0($8)
        bne   $0, $9, OSTick_ISR1
        nop
        la    $10, OSTCBCur                 /* ... save stack ptr in TCB */ 
        lw    $11, 0($10)
        sw    $sp, 0($11)
OSTick_ISR1:	
        addi  $9, $9, 1                     /* OSIntNesting++ */
	sb    $9, 0($8)
	
        mfc0  $8, $12, 0
        mfc0  $9, $13, 0
        andi  $9, $9, 0xFC00
        ins   $8, $0, 10, 6
        or    $8, $8, $9
        mtc0  $8, $12, 0
	
	jal   OSTickISR
        nop
        jal   OSIntExit
        nop
        restore_context
        eret
        nop
    .end OSTick_ISR

/***************************************************************
 *                    SPECIAL INT-SAVE CALLS
 *                  OS_STK_TYPE OSEnterCrit(void)
 *                  void OSExitCrit(OS_STK_TYPE)
 **************************************************************/
    .ent OSEnterCrit
OSEnterCrit:
        jr    $31
        di    $2
    .end OSEnterCrit

    .ent OSExitCrit
OSExitCrit:
        jr    $31
        mtc0  $4, $12, 0
    .end OSExitCrit


/***************************************************************
 *                  SPECIAL EXCEPTION VECTOR
 *                Software Interrupt 0 Handler
 *                   Jumps to OSCtxSw_ISR
 **************************************************************/
    .globl __vector_dispatch_1
    .section .vector_1,code
    .align 2
    .set nomips16
    .ent __vector_dispatch_1
__vector_dispatch_1:
	j   OSCtxSw_ISR
	nop
    .end __vector_dispatch_1

/***************************************************************
 *                   SPECIAL CORE-TIMER VECTOR
 *                  Core Timer Interrupt Handler
 *                     Jumps to OSTick_ISR
 **************************************************************/
    .globl __vector_dispatch_0
    .section .vector_0,code 
    .align 2
    .set nomips16
    .ent __vector_dispatch_0
__vector_dispatch_0:
        j   OSTick_ISR
        nop
    .end __vector_dispatch_0

/**************************************************************/
