/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_Rabbit_c.c                       *
*               Rabbit 2000/3000 Specific Code             *
*                    FAR MEMORY MODEL                      *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
    U16  *stk;
    U08   err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk              = (U16 *)pstk;                       /* Rabbit 2000/3000 Model               */
        *--stk           = (U16)data;                         /* Simulate call to function            */
        stk              = (U16 *)(((U08 *) stk) - 1);
        * (U08 *) stk    = 0;                                 /* Place XPC of 0 on stack              */
        *--stk           = (U16)task;                         /* Put pointer to task on top of stack  */
        stk              = (U16 *)(((U08 *) stk) - 1);
        * (U08 *) stk    = 0;                                 /* Place IP of 0 on stack               */
        *--stk           = (U16)0xAAAA;                       /* AF  = 0xAAAA                         */
        *--stk           = (U16)0xBBBB;                       /* BC  = 0xBBBB                         */
        *--stk           = (U16)0xDDDD;                       /* DE  = 0xDDDD                         */
        *--stk           = (U16)0x1111;                       /* HL  = 0x1111                         */
        *--stk           = (U16)0xA0AA;                       /* AF' = 0xA0AA                         */
        *--stk           = (U16)0xBBB0;                       /* BC' = 0xBBB0                         */
        *--stk           = (U16)0xDDD0;                       /* DE' = 0xDDD0                         */
        *--stk           = (U16)0x1110;                       /* HL' = 0x1110                         */
        *--stk           = (U16)0xFFFF;                       /* IX  = 0xFFFF                         */
        *--stk           = (U16)0xEEEE;                       /* IY  = 0xEEEE                         */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}
