;/***************************************************************
;/*                       pC/OS 1.87b                           *
;/*                      a FREE - RTOS                          *
;/*                                                             *
;/*                 based on the C/OS 1.00                     *
;/*                   by  Jean J. Labrosse                      *
;/*                                                             *
;/*    changed by Frank Goetze  -  www.embedded-os.de           *
;/***************************************************************
;/*                     OS_STM32F0xx_a.s                        *
;/*             STM32F0xx Cortex-M0 Specific Code               *
;/*                       using Keil                            *
;/***************************************************************

OS_NVIC_SYS_PRIx          EQU     0xE000ED20
OS_NVIC_SYS_PRIx_PENDSV   EQU     0x00C0
OS_NVIC_INT_CTRL          EQU     0xE000ED04
OS_NVIC_INT_CTRL_PENDSV   EQU     0x10000000

        IMPORT  OSTCBCur
        IMPORT  OSTCBHighRdy

    PRESERVE8
    THUMB
    AREA    |.text|, CODE, READONLY

;/***************************************************************
;/                     START MULTITASKING
;/                  void OSStartHighRdy(void)
;/***************************************************************
    EXPORT OSStartHighRdy
    THUMB
OSStartHighRdy
        LDR     R0, =OS_NVIC_SYS_PRIx      ; set PendSV-exception priority lowest
        LDRH    R1, [R0]
        LDR     R2, =OS_NVIC_SYS_PRIx_PENDSV
        ORRS    R1, R1, R2
        STRH    R1, [R0]
        MOVS    R0, #0                     ; set PSP for initial PendSV-exception
        MSR     PSP, R0
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        CPSIE   I                          ; enable interrupts
        BX      LR                         ; (for debugging)

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From task level)
;/                      void OSCtxSw(void)
;/***************************************************************
    EXPORT OSCtxSw
    THUMB
OSCtxSw
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/           PERFORM A CONTEXT SWITCH (From an ISR)
;/                    void OSIntCtxSw(void)
;/***************************************************************
    EXPORT OSIntCtxSw
    THUMB
OSIntCtxSw
        LDR     R0, =OS_NVIC_INT_CTRL      ; trigger PendSV-exception (executed after leaving interrupt) 
        LDR     R1, =OS_NVIC_INT_CTRL_PENDSV
        STR     R1, [R0]
        BX      LR

;/***************************************************************
;/                    HANDLE CONTEXT-SWITCH
;/                   void OSPendSV_ISR(void)
;/***************************************************************
;/   ==> this must be hanged into INT-table 
;/       offset-14 (PendSV handler)
;/***************************************************************
    EXPORT OSPendSV_ISR
    THUMB
OSPendSV_ISR
        MRS     R0, PSP                    ; check initial PendSV-exception
        CMP     R0, #0
        BEQ     OSPendISR_nosave           ; skip register-save the first time
        SUBS    R0, R0, #0x10
        STMIA   R0!, {R4-R7}               ; save r4-r7
        MOV     R4, R8
        MOV     R5, R9
        MOV     R6, R10
        MOV     R7, R11
        SUBS    R0, R0, #0x20
        STMIA   R0!, {R4-R7}               ; save r8-r11
        SUBS    R0, R0, #0x10
        LDR     R1, =OSTCBCur              ; save stack ptr in TCB
        LDR     R1, [R1]
        STR     R0, [R1]
OSPendISR_nosave
        LDR     R2, =OSTCBHighRdy          ; get TCB of highest priority task
        LDR     R1, =OSTCBCur              ; save as current 
        LDR     R2, [R2]
        STR     R2, [R1]
        LDR     R0, [R2]                   ; get new stack-ptr
        LDMIA   R0!, {R4-R7}               ; restore r8-r11
        MOV     R8, R4
        MOV     R9, R5
        MOV     R10, R6
        MOV     R11, R7
        LDMIA   R0!, {R4-R7}               ; restore r4-r7
        MSR     PSP, R0                    ; load PSP with new stack-ptr
        BNE     OSPendISR_LR
        MOVS    R0, #0x03
        RSBS    R0, #0                     ; ensure exception return uses process stack
        BX      R0
OSPendISR_LR
        BX      LR

;/***************************************************************
;/                       HANDLE TICK ISR
;/                     void OSTickISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************

;/***************************************************************
;/                    SPECIAL IRQ-ISR (ALL)
;/                     void OSirqISR(void)
;/***************************************************************
;/                       not needed here
;/***************************************************************


;/***************************************************************
;/                    SPECIAL INT-SAVE CALLS
;/                  OS_STK_TYPE OSEnterCrit(void)
;/                  void OSExitCrit(OS_STK_TYPE)
;/***************************************************************
    EXPORT OSEnterCrit
    THUMB
OSEnterCrit
        MRS     R0, PRIMASK
        CPSID   I
        BX      LR                          ; return R0

    EXPORT OSExitCrit
    THUMB
OSExitCrit
        MSR     PRIMASK, R0
        BX      LR                          ; return

;/***************************************************************
    ALIGN 4
    END


