;****************************************************************
;*                       pC/OS 1.70b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                     OS_STR711_a.s79                          *
;*                STR711-ARM Specific Code                      *
;*                        using IAR                             *
;****************************************************************
            EXTERN  OSTCBCur
            EXTERN  OSTCBHighRdy
            EXTERN  OSIntExit
            EXTERN  OSIntNesting
            
            EXTERN  IntVectorTable

NO_INT      EQU     0xC0                    ; mask to disable interrupts
FIQ32_MODE  EQU     0x11
IRQ32_MODE  EQU     0x12
SVC32_MODE  EQU     0x13

EIC_base_addr        EQU    0xFFFFF800      ; EIC base address.
CICR_off_addr        EQU    0x04            ; Current Interrupt Channel Register.
IVR_off_addr         EQU    0x18            ; Interrupt Vector Register.
IPR_off_addr         EQU    0x40            ; Interrupt Pending Register.
IVR_mask             EQU    0xFFFF

        RSEG    CODE:CODE:NOROOT(2)

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
        PUBLIC  OSStartHighRdy
        CODE32
OSStartHighRdy
        MSR     CPSR_cxsf, #0xD3            ; switch to SVC mode with IRQ and FIQ disabled
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore stack-ptr
        LDR     R4, [SP], #4                ; pop task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop task context

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
        PUBLIC  OSCtxSw
        CODE32
OSCtxSw
        STMFD   SP!, {LR}                   ; push return address
        STMFD   SP!, {LR}
        STMFD   SP!, {R0-R12}               ; push registers
        MRS     R4, CPSR                    ; push current CPSR
        TST     LR, #1                      ; see if called from Thumb mode
        ORRNE   R4, R4, #0x20               ; if yes, Set the T-bit
        STMFD   SP!, {R4}
        LDR     R4, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore new stack-ptr
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
        PUBLIC  OSIntCtxSw
        CODE32
                                            ; no TCB-save since the INTs can cascaded
OSIntCtxSw                                  ; (only saved on first INT by OSirqISR or OSfiqISR)
        LDR     R6, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R4, =OSTCBCur               ; save as current
        LDR     R6, [R6]
        STR     R6, [R4]
        LDR     SP, [R6]                    ; restore new stack-ptr
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
;   ==> this is not asm-relevant for this processor
;       the C-TimerISR is to hang into the table of
;       the following IRQ-isr
;****************************************************************


;****************************************************************
;                     SPECIAL IRQ-ISR (ALL)
;                      void OSirqISR(void)
;****************************************************************
        PUBLIC  OSirqISR
        CODE32
OSirqISR
        STMFD   SP!, {R1-R3}                ; push working registers onto IRQ stack
        MOV     R1, SP                      ; save IRQ stack pointer
        ADD     SP, SP,#12                  ; adjust IRQ stack pointer
        SUB     R2, LR,#4                   ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from IRQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1 ...
        BNE     OSirqISR1
        LDR     R4, =OSTCBCur               ; ... Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSirqISR1:
        MSR     CPSR_c, #(NO_INT | IRQ32_MODE) ; change to IRQ mode (to use the IRQ stack to handle interrupt)
;--- HW specific ---
        LDR     R0, =EIC_base_addr
        LDR     R0, [R0, #IVR_off_addr]
        LDR     R1, =IVR_mask
        AND     R0, R0, R1
        MOV     R0, R0, LSL #4
        LDR     R1, =IntVectorTable         ; Branch to the IRQ handler.
        LDR     R0, [R1, R0]
;-------------------
        MOV     LR, PC
        BX      R0
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
;--- HW specific ---
        LDR    R0, =EIC_base_addr           ; Clear pending bit in EIC (using the proper IPRx)
        LDR    R2, [R0, #CICR_off_addr]     ; Get the IRQ channel number.
        MOV    R3, #1
        MOV    R3, R3, LSL R2
        STR    R3, [R0, #IPR_off_addr]      ; Clear the corresponding IPR bit.
;-------------------
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context

;****************************************************************
;                     SPECIAL FIQ-ISR (ALL)
;                      void OSfiqISR(void)
;****************************************************************
        PUBLIC  OSfiqISR
        CODE32
OSfiqISR
        STMFD   SP!, {R1-R3}                ; push working registers onto FIQ stack
        MOV     R1, SP                      ; save FIQ stack pointer
        ADD     SP, SP,#12                  ; adjust FIQ stack pointer
        SUB     R2, LR,#4                   ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from FIQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1,#1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1
        BNE     OSfiqISR1
        LDR     R4, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSfiqISR1:
        MSR     CPSR_c, #(NO_INT | FIQ32_MODE) ; change to FIQ mode (to use the IRQ stack to handle interrupt)
;--- HW specific ---
        LDR     R0, =EIC_base_addr
        LDR     R0, [R0, #IVR_off_addr]
        LDR     R1, =IVR_mask
        AND     R0, R0, R1
        MOV     R0, R0, LSL #4
        LDR     R1, =IntVectorTable         ; Branch to the IRQ handler.
        LDR     R0, [R1, R0]
;-------------------
        MOV     LR, PC
        BX      R0
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
;--- HW specific ---
        LDR    R0, =EIC_base_addr           ; Clear pending bit in EIC (using the proper IPRx)
        LDR    R2, [R0, #CICR_off_addr]     ; Get the IRQ channel number.
        MOV    R3, #1
        MOV    R3, R3, LSL R2
        STR    R3, [R0, #IPR_off_addr]      ; Clear the corresponding IPR bit.
;-------------------
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context

;****************************************************************
;                     SPECIAL INT-SAVE CALLS
;                   OS_STK_TYPE OSEnterCrit(void)
;                   void OSExitCrit(OS_STK_TYPE)
;****************************************************************
        PUBLIC  OSEnterCrit
        CODE32
OSEnterCrit
        MRS     R0, CPSR
        ORR     R1, R0, #0x80
        MSR     CPSR_c, R1
        ORR     R1, R1, #0x40
        MSR     CPSR_c, R1
        BX      LR                          ; return R0

        PUBLIC  OSExitCrit
        CODE32
OSExitCrit
        MSR     CPSR_c, r0
        BX      LR                          ; return


;****************************************************************
        END


