/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                     OS_STR711_c.c                        *
*                   ARM Specific Code                      *
*                       (ARM/THUMB)                        *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

#define  ARM_MODE_ARM     0x00000000
#define  ARM_MODE_THUMB   0x00000020

#ifdef __IAR_SYSTEMS_ICC__
 #if __CPU_MODE__ == 1
  #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
 #else
  #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
 #endif
#else
 #ifdef __KEIL__
  #ifdef __THUMB__
   #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
  #else
   #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
  #endif
 #else
  #ifdef __IMAGECRAFT__
   #ifdef THUMB
    #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
   #else
    #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
   #endif
  #else
   #ifdef __GNUC__
    #ifdef __thumb__              // or #ifndef __arm__
     #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
    #else
     #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
    #endif
   #else
    #ifdef __CC_ARM
     #ifdef __APCS_INTERWORK
      #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_THUMB)
     #else
      #define  ARM_SVC_MODE    (0x00000013L + ARM_MODE_ARM)
     #endif
    #endif
   #endif
  #endif
 #endif
#endif

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U32         *stk;
    U08          err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk      = (U32 *)pstk;                               /* decrement first !                    */
        *--stk   = (U32)(task);                               /* task start address                   */
        *--stk   = (U32)0x14141414L;                          /* R14 (LR)                             */
        *--stk   = (U32)0x12121212L;                          /* R12                                  */
        *--stk   = (U32)0x11111111L;                          /* R11                                  */
        *--stk   = (U32)0x10101010L;                          /* R10                                  */
        *--stk   = (U32)0x09090909L;                          /* R9                                   */
        *--stk   = (U32)0x08080808L;                          /* R8                                   */
        *--stk   = (U32)0x07070707L;                          /* R7                                   */
        *--stk   = (U32)0x06060606L;                          /* R6                                   */
        *--stk   = (U32)0x05050505L;                          /* R5                                   */
        *--stk   = (U32)0x04040404L;                          /* R4                                   */
        *--stk   = (U32)0x03030303L;                          /* R3                                   */
        *--stk   = (U32)0x02020202L;                          /* R2                                   */
        *--stk   = (U32)0x01010101L;                          /* R1                                   */
        *--stk   = (U32)Tdata;                                /* R0 : argument                        */
        *--stk   = (U32)ARM_SVC_MODE;                         /* CPSR  (Enable IRQ and FIQ)           */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

