;****************************************************************
;*                       pC/OS 1.87b                            *
;*                      a FREE - RTOS                           *
;*                                                              *
;*                 based on the C/OS 1.00                      *
;*                   by  Jean J. Labrosse                       *
;*                                                              *
;*    changed by Frank Goetze  -  www.embedded-os.de            *
;****************************************************************
;*                      OS_TMS570_a.s                           *
;*               ARM Cortex-R4 Specific Code                    *
;*                        using IAR                             *
;****************************************************************

            EXTERN  OSTCBCur
            EXTERN  OSTCBHighRdy
            EXTERN  OSIntExit
            EXTERN  OSIntNesting
            
            EXTERN  miniVIM_irq_dispatch
            EXTERN  miniVIM_fiq_dispatch

NO_INT      EQU     0xC0                    ; mask to disable interrupts
I_bit       EQU     0x80
F_bit       EQU     0x40
FIQ32_MODE  EQU     0x11
IRQ32_MODE  EQU     0x12
SVC32_MODE  EQU     0x13

            RSEG    CODE:CODE:NOROOT(2)

;****************************************************************
;                      START MULTITASKING
;                   void OSStartHighRdy(void)
;****************************************************************
        PUBLIC  OSStartHighRdy
        CODE32
OSStartHighRdy
        MSR     CPSR_c, #0xD3               ; switch to SVC mode with IRQ and FIQ disabled
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore stack-ptr
        LDMFD   SP!, {R0}                   ; pop task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop task context
        BX      LR

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From task level)
;                       void OSCtxSw(void)
;****************************************************************
        PUBLIC  OSCtxSw
        CODE32
OSCtxSw
        STMFD   SP!, {LR}                   ; push return address
        STMFD   SP!, {R0-R12,LR}            ; push LR & registers
        MRS     R0, CPSR                    ; push current CPSR
        TST     LR, #1                      ; see if called from Thumb mode
        ORRNE   R0, R0, #0x20               ; if yes, Set the T-bit
        STMFD   SP!, {R0}
        LDR     R0, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R1, [R0]
        STR     SP, [R1]
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore new stack-ptr
        LDMFD   SP!, {R0}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        BX      LR

;****************************************************************
;            PERFORM A CONTEXT SWITCH (From an ISR)
;                     void OSIntCtxSw(void)
;****************************************************************
        PUBLIC  OSIntCtxSw
        CODE32                              ; no TCB-save since the INTs can cascaded
OSIntCtxSw                                  ; (only saved on first INT by OSirqISR or OSfiqISR)
        LDR     R1, =OSTCBHighRdy           ; get TCB of highest priority task
        LDR     R0, =OSTCBCur               ; save as current
        LDR     R1, [R1]
        STR     R1, [R0]
        LDR     SP, [R1]                    ; restore new stack-ptr
        LDMFD   SP!, {R0}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R0
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        BX      LR

;****************************************************************
;                        HANDLE TICK ISR
;                      void OSTickISR(void)
;****************************************************************
;   ==> this is not asm-relevant for this processor
;       the C-TimerISR is to hang into the table of
;       the following IRQ-isr
;****************************************************************


;****************************************************************
;                     SPECIAL IRQ-ISR (ALL)
;                      void OSirqISR(void)
;****************************************************************
        PUBLIC  OSirqISR
        CODE32
OSirqISR
        STMFD   SP!, {R1-R3}                ; push working registers onto IRQ stack
        MOV     R1, SP                      ; save IRQ stack pointer
        ADD     SP, SP, #12                 ; adjust IRQ stack pointer
        SUB     R2, LR, #4                  ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from IRQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1, #1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1 ...
        BNE     OSirqISR1
        LDR     R4, =OSTCBCur               ; ... Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSirqISR1:
        MSR     CPSR_c, #(NO_INT | IRQ32_MODE) ; change to IRQ mode (to use the IRQ stack to handle interrupt)
;--- HW specific ---
        BLX     miniVIM_irq_dispatch          ; Branch to the indexed interrupt
;-------------------
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        BX      LR

;****************************************************************
;                     SPECIAL FIQ-ISR (ALL)
;                      void OSfiqISR(void)
;****************************************************************
        PUBLIC  OSfiqISR
        CODE32
OSfiqISR
        STMFD   SP!, {R1-R3}                ; push working registers onto FIQ stack
        MOV     R1, SP                      ; save FIQ stack pointer
        ADD     SP, SP, #12                 ; adjust FIQ stack pointer
        SUB     R2, LR, #4                  ; adjust PC for return address to task
        MRS     R3, SPSR                    ; copy SPSR (i.e. interrupted tasks CPSR)
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        STMFD   SP!, {R2}                   ; push tasks return PC
        STMFD   SP!, {LR}                   ; push tasks LR
        STMFD   SP!, {R4-R12}               ; push tasks R12-R4
        LDMFD   R1!, {R4-R6}                ; move tasks R1-R3 from FIQ stack to SVC stack
        STMFD   SP!, {R4-R6}
        STMFD   SP!, {R0}                   ; push task R0 onto task stack
        STMFD   SP!, {R3}                   ; push task CPSR (i.e. IRQ SPSR)
        LDR     R0, =OSIntNesting           ; OSIntNesting++ (don't call OSIntEnter !)
        LDRB    R1, [R0]
        ADD     R1, R1, #1
        STRB    R1, [R0]
        CMP     R1, #1                      ; if OSIntNesting == 1
        BNE     OSfiqISR1
        LDR     R4, =OSTCBCur               ; Save stack ptr in TCB
        LDR     R5, [R4]
        STR     SP, [R5]
OSfiqISR1:
        MSR     CPSR_c, #(NO_INT | FIQ32_MODE) ; change to FIQ mode (to use the IRQ stack to handle interrupt)
;--- HW specific ---
        BLX     miniVIM_fiq_dispatch          ; Branch to the indexed interrupt
;-------------------
        MSR     CPSR_c, #(NO_INT | SVC32_MODE) ; change to SVC mode
        LDR     R0, =OSIntExit              ; call OSIntExit() -> perform a context-switch if nesesary
        MOV     LR, PC
        BX      R0
        LDMFD   SP!, {R4}                   ; pop new task CPSR
        MSR     SPSR_cxsf, R4
        LDMFD   SP!, {R0-R12,LR,PC}^        ; pop new task context
        BX      LR

;****************************************************************
;                     SPECIAL INT-SAVE CALLS
;                   OS_STK_TYPE OSEnterCrit(void)
;                   void OSExitCrit(OS_STK_TYPE)
;****************************************************************
        PUBLIC  OSEnterCrit
        CODE32
OSEnterCrit
        MRS     R0, CPSR
        ORR     R1, R0, #I_bit
        MSR     CPSR_c, R1
        ORR     R1, R1, #F_bit
        MSR     CPSR_c, R1
        BX      LR                          ; return R0

        PUBLIC  OSExitCrit
        CODE32
OSExitCrit
        MSR     CPSR_c, R0
        BX      LR                          ; return


;****************************************************************
        END


