/*************************************************************
*
* Description:
*   TMS570 - mini VIM support for pC/OS RTOS kernel
*            to get the central IRQ-entry functional
*   This is a minimalistic VIM implementation w/o the TMS570
*   system will not support IRQ/FIQ event system
*
* Author[s]:
*   Frank Goetze
**************************************************************/

#include <TMS570.h>
#include "miniVIM.h"

static miniVIM_ram_t *vimVEC = (volatile miniVIM_ram_t *) 0xFFF82000;
static miniVIM_t     *vimREG = (volatile miniVIM_t     *) 0xFFFFFDEC;

void miniVIM_dummy_interrupt(void)
{
    // dummy IRQ handler
}

void miniVIM_set_vector(unsigned vector_nr, VecFuncPTR fkt)
{
    vimVEC->ISR[vector_nr] = fkt;
}

void miniVIM_enable_int(unsigned vector_nr)
{
    vimREG->REQENASET[vector_nr >> 5] = 1 << (vector_nr & 0x1f);
}

void miniVIM_disable_int(unsigned vector_nr)
{
    vimREG->REQENACLR[vector_nr >> 5] = 1 << (vector_nr & 0x1f); 
}

void miniVIM_irq_dispatch(void)
{
    (*vimVEC->ISR[vimREG->IRQINDEX-1])();
}

void miniVIM_fiq_dispatch(void)
{
    vimVEC->ISR[vimREG->FIQINDEX-1];  
}

void miniVIM_Init(void)
{
    unsigned i, value = 0x00010203; 
    
    /* Disable interrupts */ 
    for (i = 0; i < 3; i++) vimREG->REQENACLR[i] = 0xffffffff;
    
    /* Map default events on interrupt vectors */ 
    for (i = 0; i < 24; i += 1, value += 0x04040404) vimREG->CHANCTRL[i] = value;
 
    /* Set all vectors as IRQ (not FIR) */ 
    vimREG->FIRQPR[0] = 3; vimREG->FIRQPR[1] = 0; vimREG->FIRQPR[2] = 0; 

    /* Set all vectors to dummy handler */
    for (i = 0; i <= 94; ++i) vimVEC->ISR[i] = miniVIM_dummy_interrupt; 

    /* Clear bit VE in SCTLR register to not use VIM IRQ exception bypass*/ 
#ifdef __IAR_SYSTEMS_ICC__
    asm volatile ("mrc p15, 0, %0, c1, c0, 0\n": "=r" (value));
    value &= ~(1 << 24); 
    asm volatile ("mcr p15, 0, %0, c1, c0, 0\n": : "r" (value));
#else
    register unsigned int cp15_control __asm("cp15:0:c1:c0:0");
    cp15_control &= ~(1 << 24);
#endif
}

/*************************************************************/
