/*************************************************************
*
* Description:
*	Z182 - Example program using Softools WinIDE
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../../inc/pC_OS.h"

#define	TASK_STK_SIZE   512

extern void OSTickISR(void);

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];

U08 pool[2048];

OS_MUX Mutex1;
OS_EVG Events1;

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

void main()
{
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), 2048);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 12);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 12);
#endif
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);

    load_I_register(0x3F);
    IL = 0x00;

    OS_Start();
}

void AppTickInit(void)
{
    U08 tmp;
    
    tmp = input(TCR) & 0xEE;
    output(TCR, tmp);
    tmp = input(TMDR0H);
    output(TMDR0L, 0xFF);
    output(TMDR0H, 0x3B);
    output(RLDR0L, 0xFF);
    output(RLDR0H, 0x3B);
    tmp = input(TCR) | 0x11;
    output(TCR, tmp);
}

void Task1(void *Tdata)
{
    U08 returnOk;
    U32 event, i;
    U08 OS_HUGE *ptr;

    Tdata = Tdata;
    i=0;
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(10);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}


void Task2(void *Tdata)
{
    U08 returnOk;
    U32 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);     // suspending
        if(!returnOk) {
            i=0;
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(10);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
		}
    }
}

void Task3(void *Tdata)
{
    U16 i;

    Tdata = Tdata;
    i=0;
    while(1) {
        OS_TimeDly(10);                                 // for Task2
        i++;
    }
}
