/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
*                      OS_Rabbit_c.c                       *
*                   Z-180 Specific code                    *
*                   BANKED MEMORY MODEL                    *
***********************************************************/
#include "../../inc/pC_OS.h"
#include "../../inc/_private/pC_OS_internal.h"

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
#if OS_STK_CHECK_EN
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, void *pstkend, U08 p)
#else
U08 OS_TaskCreate(void (OS_FAR *task)(void *dptr), void *Tdata, void *pstk, U08 p)
#endif
{
    U16  *stk;
    U16   task_bank;
    U16   task_offset;
    U08   err;

    if (p >= OS_MIN_PRIO)
        return (OS_PRIO_INVALID);
    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                     /* Avoid creating task if already exist */
        OSTCBPrioTbl[p] = (OS_TCB *)1;
        OS_EXIT_CRITICAL();
        stk              = (U16 *)pstk;                       /* Z180 Banked Model                    */
        task_bank        = (U16)((U32)task >> 16);
        task_bank        = (task_bank << 8) + task_bank;
        task_offset      = (U16)((U32)task & 0xFFFFL);    
        *--stk           = (U16)task_bank;                    /* Save bank repeated in upper and lower register        */
        *--stk           = (U16)task_offset;                  /* Save task address                    */
        *--stk           = (U16)task_offset;                  /* Saved PC from ISR (Simulated)        */
        *--stk           = (U16)0x1100;                       /* AF   Primary registers               */
        *--stk           = (U16)0x2222;                       /* BC                                   */
        *--stk           = (U16)pdata;                        /* DE = pdata is passed in DE           */
        *--stk           = (U16)0x4444;                       /* HL                                   */
        *--stk           = (U16)0x5555;                       /* AF   Alternate registers             */
        *--stk           = (U16)0x6666;                       /* BC                                   */
        *--stk           = (U16)0x7777;                       /* DE                                   */
        *--stk           = (U16)0x8888;                       /* HL                                   */
        *--stk           = (U16)0x9999;                       /* IX                                   */
        *--stk           = (U16)0xAAAA;                       /* IY                                   */
        *--stk           = (U16)task_bank;                    /* Save bank (repeated in upper and lower register) */
#if OS_STK_CHECK_EN
        err = OSTCBInit(p, (void OS_FAR *)stk, (void OS_FAR *)pstkend);
#else
        err = OSTCBInit(p, (void OS_FAR *)stk);
#endif
        if (err == OS_NO_ERR) {
            if (OSRunning)
                OSSched();
        } else
            OSTCBPrioTbl[p] = (OS_TCB *)0;
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}
