/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                     pC_OS_queue.c                        *
*                     QUEUE SERVICE                        *
***********************************************************/

#if OS_Q_EN
/*
************************************************************
*                    INITIALIZE QUEUE                      *
************************************************************
*/
U08 OS_QueueInit(OS_Q *pq, void OS_HUGE *buffer, U16 size) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          i;

    OS_ENTER_CRITICAL();
    pq->OSQData     = (U08 OS_HUGE *) buffer;
    pq->OSQIn       = 0;
    pq->OSQOut      = 0;
    pq->OSQSize     = size;
    pq->OSQEntries  = 0;
    pq->OSQFGrp     = 0x00;
    pq->OSQBGrp     = 0x00;
    for (i = 0; i < OS_PRIO_TBL; i++) {
        pq->OSQFTbl[i]  = 0x00;
        pq->OSQBTbl[i]  = 0x00;
    }
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                   GET INFO OF QUEUE                      *
************************************************************
*/
U08 OS_QueueInfo(OS_Q *pq, U16 *size, U16 *used, U08 *prio) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y;

    OS_ENTER_CRITICAL();
    *size = pq->OSQSize;
    *used = pq->OSQEntries;
    if (pq->OSQFGrp != 0) {
        y    = OSUnMapTbl[pq->OSQFGrp];
        x    = OSUnMapTbl[pq->OSQFTbl[y]];
       *prio = (y << 3) + x;
    } else
       *prio = 0;
    OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                 CLEAR ALL DATA OF QUEUE                  *
************************************************************
*/
U08 OS_QueueClear(OS_Q *pq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    pq->OSQIn       = 0;
    pq->OSQOut      = 0;
    pq->OSQEntries  = 0;
    if (pq->OSQBGrp != 0) {
        y                = OSUnMapTbl[pq->OSQBGrp];
        x                = OSUnMapTbl[pq->OSQBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
            pq->OSQBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     PEND ON A QUEUE                      *
************************************************************
*/
U08 OS_QueuePend(OS_Q *pq, U08 OS_FAR *msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pq->OSQEntries == 0) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_Q_NODATA);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_Q;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pq->OSQFTbl[y] |= bitx;
        pq->OSQFGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if ((OSTCBCur->OSTCBStat & OS_STAT_Q) && (pq->OSQEntries == 0)){
            if ((U08)(pq->OSQFTbl[y] &= ~(bitx)) == 0)
                pq->OSQFGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            msg[0]              = 0;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (pq->OSQEntries == 0) {
            OS_EXIT_CRITICAL();
            return (OS_Q_NODATA);
        }
    }
    msg[0] = pq->OSQData[pq->OSQOut++];
    pq->OSQEntries--;
    if (pq->OSQOut == pq->OSQSize)
        pq->OSQOut = 0;
    if (pq->OSQBGrp != 0) {
        y                = OSUnMapTbl[pq->OSQBGrp];
        x                = OSUnMapTbl[pq->OSQBTbl[y]];
        bity             = OSMapTbl[y];
        bitx             = OSMapTbl[x];
        if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
            pq->OSQBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSTCBPrioTbl[p]->OSTCBDly   =  0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                     POST TO A QUEUE                      *
************************************************************
*/
U08 OS_QueuePost(OS_Q *pq, U08 msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pq->OSQEntries >= pq->OSQSize) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_Q_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_Q;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pq->OSQBTbl[y] |= bitx;
        pq->OSQBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (OSTCBCur->OSTCBStat & OS_STAT_Q) {
            if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
                pq->OSQBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (pq->OSQEntries >= pq->OSQSize) {
            OS_EXIT_CRITICAL();
            return (OS_Q_FULL);
        }
    }
    pq->OSQData[pq->OSQIn++] = msg;
    pq->OSQEntries++;
    if (pq->OSQIn == pq->OSQSize)
        pq->OSQIn = 0;
    if (pq->OSQFGrp != 0) {
        y    = OSUnMapTbl[pq->OSQFGrp];
        x    = OSUnMapTbl[pq->OSQFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pq->OSQFTbl[y] &= ~(bitx)) == 0)
            pq->OSQFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*                POST TO FRONT OF A QUEUE                  *
************************************************************
*/
U08 OS_QueueFrontPost(OS_Q *pq, U08 msg, U16 timeout) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pq->OSQEntries >= pq->OSQSize) {
        if ((timeout == OS_NO_SUSP) || (OSIntNesting > 0)) {
            OS_EXIT_CRITICAL();
            return (OS_Q_FULL);
        }
#if OS_TASK_DESTROY_EN
        OSTCBCur->OSTCBipc   = pq;
#endif
        OSTCBCur->OSTCBStat |= OS_STAT_Q;
        OSTCBCur->OSTCBDly   = timeout;
        y                    = OSTCBCur->OSTCBPrio >> 3;
        x                    = OSTCBCur->OSTCBPrio & 0x07;
        bity                 = OSMapTbl[y];
        bitx                 = OSMapTbl[x];
        if ((OSRdyTbl[y] &= ~(bitx)) == 0)
            OSRdyGrp &= ~(bity);
        pq->OSQBTbl[y] |= bitx;
        pq->OSQBGrp    |= bity;
        OS_EXIT_CRITICAL();
        OSSched();
        OS_ENTER_CRITICAL();
        if (OSTCBCur->OSTCBStat & OS_STAT_Q) {
            if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
                pq->OSQBGrp &= ~(bity);
            OSTCBCur->OSTCBStat = OS_STAT_RDY;
            OS_EXIT_CRITICAL();
            return (OS_TIMEOUT);
        }
        if (pq->OSQEntries >= pq->OSQSize) {
            OS_EXIT_CRITICAL();
            return (OS_Q_FULL);
        }
    }
    pq->OSQOut--;
    if (pq->OSQOut >= pq->OSQSize)
        pq->OSQOut = pq->OSQSize-1;
    pq->OSQData[pq->OSQOut] = msg;
    pq->OSQEntries++;
    if (pq->OSQFGrp != 0) {
        y    = OSUnMapTbl[pq->OSQFGrp];
        x    = OSUnMapTbl[pq->OSQFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pq->OSQFTbl[y] &= ~(bitx)) == 0)
            pq->OSQFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
    } else
        OS_EXIT_CRITICAL();
    return (OS_NO_ERR);
}
/*
************************************************************
*             ABBORT HIGHEST PENDER ON A QUEUE             *
************************************************************
*/
U08 OS_QueuePendAbbort(OS_Q *pq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pq->OSQFGrp != 0) {
        y    = OSUnMapTbl[pq->OSQFGrp];
        x    = OSUnMapTbl[pq->OSQFTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pq->OSQFTbl[y] &= ~(bitx)) == 0)
            pq->OSQFGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}
/*
************************************************************
*             ABBORT HIGHEST POSTER ON A QUEUE             *
************************************************************
*/
U08 OS_QueuePostAbbort(OS_Q *pq) KC51_REENTRANT
{
#if OS_CRITICAL_SR
    OS_STK_TYPE  OS_sr;
#endif
    U08          x, y, bitx, bity, p;

    OS_ENTER_CRITICAL();
    if (pq->OSQBGrp != 0) {
        y    = OSUnMapTbl[pq->OSQBGrp];
        x    = OSUnMapTbl[pq->OSQBTbl[y]];
        bity = OSMapTbl[y];
        bitx = OSMapTbl[x];
        if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
            pq->OSQBGrp &= ~(bity);
        p                           = (y << 3) + x;
        OSTCBPrioTbl[p]->OSTCBDly   = 0;
        OSTCBPrioTbl[p]->OSTCBStat &= ~(OS_STAT_Q);
        OSRdyGrp                   |= bity;
        OSRdyTbl[y]                |= bitx;
        OS_EXIT_CRITICAL();
        OSSched();
        return (OS_NO_ERR);
    }
    OS_EXIT_CRITICAL();
    return (OS_TIME_NOT_DLY);
}

#if OS_TASK_DESTROY_EN
/*
************************************************************
*                 DELETE A TASK FROM A QUEUE               *
************************************************************
*/
void OSQueueDestroyTask(OS_Q *pq, U08 prio)
{
    U08          x, y, bitx, bity;

    y      = prio >> 3;
    x      = prio & 0x07;
    bity   = OSMapTbl[y];
    bitx   = OSMapTbl[x];
    if ((U08)(pq->OSQFTbl[y] &= ~(bitx)) == 0)
        pq->OSQFGrp &= ~(bity);
    if ((U08)(pq->OSQBTbl[y] &= ~(bitx)) == 0)
        pq->OSQBGrp &= ~(bity);
    OSTCBPrioTbl[prio]->OSTCBDly   =  0;
    OSTCBPrioTbl[prio]->OSTCBStat &= ~(OS_STAT_Q);
    OSRdyGrp                      |=  bity;
    OSRdyTbl[y]                   |=  bitx;
}
#endif /* OS_TASK_DESTROY_EN */
#endif /* OS_Q_EN */

/************************* END ****************************/


