/***********************************************************
*                       pC/OS 1.87b                        *
*                   a FREE - RTOS Kernel                   *
*                                                          *
*                 based on the C/OS 1.00                  *
*                   by  Jean J. Labrosse                   *
*                                                          *
*      changed by Frank Goetze  -  www.embedded-os.de      *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                        pC_OS.c                           *
*                   SERVICE COLLECTION                     *
************************************************************
* Revision-Info:                                           *
*                                                          *
* 1.00a     changes in Messagebox, Semaphore and Queue     *
*           ( comment: no task get a pointer into the      *
*             data-area of another task )                  *
*           Pipes, Eventgroups, dyn. Memoryman. added      *
* 1.00b     Timestamp-mark, History, DlyResume, Task-      *
*           Supension, QueueFrontPost added                *
*           some bugfixes                                  *
* 1.00c     change Pipe for faster copy one paket          *
*           ( if waiting task higher and no data in this   *
              pipe )                                       *
*           change Posting for wait or suspend             *
* 1.00d     implement correct waiting while posting,       *
*           add ClearQueue                                 *
* 1.00e     implement OS_FAR and OS_HUGE for flexible      *
*           memory-model, change Init of Task and others   *
*           for compatibility to older uC/OS-ports,        *
*           OS_QueueInfo and OS_PipeInfo added             *
* 1.00f     bugfix in Eventgroup and OS_SemAccept added    *
* 1.00g     compiler-switches added, bugfix in semaphore   *
* 1.00h     MUTEXES (priority ceilings) added, some        *
*           IntNesting-checks again timewait added,        *
*           some cleanup's and complete redesign of        *
*           EventGroups                                    *
* 1.40a     some C-fixes and ATMega128 added,              *
*           ==> name and revision changed <==              *
* 1.41a     OS_STK_TYPE as type of stack entry added,      *
*           small correction in OSEvgLink()                *
* 1.50c     complete update to U08..U32 types,             *
*           implementation of OS_PACKED, OS_PACKED_ATTR    *
*           and align(x)-corrections, small correction in  *
*           OS_TaskSuspend()                               *
* 1.52b     split min-prio and max-tasks to reduce RAM     *
*           on smaller max-tasks, using "NULL" as pointer, *
*           add unique task-ID for other modules on use    *
*           of prio-changes and to reduce RAM on this      *
*           modules                                        *
* 1.60c     add temporary reg-holder for disabling         *
*           interrupts as protection again neested ints    *
* 1.61a     memory manager alltimes work MPU aligned       *
* 1.65b     TIMERS (using a timer-task) added, some small  *
*           internal changes for differ system & timers,   *
*           OS_TaskDelete changed, OS_TaskChangePrio added *
*           OS_...PendAbbort and OS_...PostAbbort added,   *
*           some small internal fixes                      *
* 1.66c     some checks added in TIMERS, implementation of *
*           OS_TimerGetState() and OS_TimerGetRemain()     *
* 1.67a     reducing RAM consumption on prio-tables of     *
*           IPC-recources (SEM/EVG/MUX/Queue/Pipe/..)      *
* 1.68a     OS_EvgPendAbbort() added                       *
* 1.70a     optional simple debug feature OS_STK_CHECK_EN  *
*           added, checking "end-of-stack was used" -      *
*           checked on shedule to an other task            *
* 1.70b     reorg components/user/cpu configuration        *
* 1.71a     add optional stack-fill to stack-check feature *
* 1.80c     some fixes in all ...Abbort() functions,       *
*           some additional checks for "mutex in use",     *
*           OS_TaskGetID() and OS_TaskGetStatus() added,   *
*           with new OS_TaskDestroy() now tasks, waiting   *
*           on a resource, can deleted too,                *
*           source file splitted for better handling       *
* 1.81b     task-ID to user-callback in OS_TaskDestroy()   *
*           added to decode used mutex and user for reinit *
*           recource and optional user-handles             *
* 1.81c     OS_TaskDestroy() do not accept OS_PRIO_SELF    *
* 1.82a     OS_TaskDestroy() renamed to OS_TaskIdDestroy() *
*           OS_TaskGetPrio(), OS_TimeDelayIdResume(),      *
*           OS_TaskIdDelete(), OS_TaskIdGetStatus(),       *
*           OS_TaskIdSyspend() and OS_TaskIdResume() added *
*           some additional checks for prio and/or id      *
* 1.83a     small fix for OS_TaskIdDestroy() in            *
*           OSIntExit(), if the current running task       *
*           destroyed by an ISR                            *
* 1.83b     extra small fix for missing compiler switch    *
*           in OSTCBInit(..)                               *
* 1.84a     change events param in eventgroups to pointer  *
*           for better handling of OR mode (with return)   *
* 1.85b     correction in eventgroups for multiple         *
*           pending tasks on one eventgroup                *
* 1.86a     OS_ID_SELF added for OS_TaskIdDelete() and     *
*           OS_TaskGetPrio()                               *
* 1.87b     an interrupt and during sheduling is switched  *
*           off a sender can't use the fast mode on        *
*           OS_Pipe...Post since data source can't be      *
*           guarantied                                     *
*                                                          *
***********************************************************/

#include "../inc/pC_OS.h"
#include "../inc/_private/pC_OS_internal.h"
/*
************************************************************
*                    TABLES IN CODE                        *
************************************************************
*/
U08 OS_CONST OS_ROMDATA OSRevision[] = {"pC/OS 1.87b"};
/*
************************************************************
*                 COLLECTION OF SERVICES                   *
************************************************************
*/
#include "_public/pC_OS_userCB.c"  // import USER CALLBACKS
#include "_private/pC_OS_tmr.c"    // import TIMER       service
#include "_private/pC_OS_task.c"   // import TASK        service
#include "_private/pC_OS_sem.c"    // import SEMAPHORE   service
#include "_private/pC_OS_mutex.c"  // import MUTEX       service
#include "_private/pC_OS_mbox.c"   // import MESSAGEBOX  service
#include "_private/pC_OS_queue.c"  // import QUEUE       service
#include "_private/pC_OS_pipe.c"   // import PIPE        service
#include "_private/pC_OS_evg.c"    // import EVENTGROUP  service
#include "_private/pC_OS_mem.c"    // import MEMORY      service
#include "_private/pC_OS_his.c"    // import HISTORY     service
/*
************************************************************
*                    GET REVISION                          *
************************************************************
*/
U08 OS_FAR *OS_GetRev(void) KC51_REENTRANT
{
    return((U08 OS_FAR *)OSRevision);
}

/************************* END ****************************/


